 pro spline_interp,res,gminf,gmin_header,qc_threshold


;##########################################################################
;# Cubic spline interpolation (Natural Spline)                            #
;#                                                                        #
;# (1) read event data onto each rainy minute                             #
;# (2) QC data on event basis, discard bad event                          #
;# (3) do Cubic Spline interpolation if rain event duration >1 min        #
;# (4) estimate the beginning and end minutes (bmin & emin)               #
;#     for each event based on Cubic Spline extrapolation                 #
;# (5) interpolate rain from bmin to emin using spline                    #
;# (6) evenly spread one minute event into 5 min interval from            #
;#     (time stamp at minute-3) to (time stamp at minute+1)               #  
;# (7) linearly interpolate rain from (1st minute on time stamp-4)        #
;#     to last minute if spline bias>1.5 or <0.5                          #
;# (8) write 1-min rainrates to gmin files                                #
;#                                                                        #
;# Written by Jianxin Wang: NASA/GSFC/612/SSAI                            #  
;#                                                                        #
;##########################################################################
;
;-----------------------------------------------------------------------------

 
openw,  lun_gmin, gminf, /get_lun 
printf, lun_gmin, gmin_header

halftip=res/2.0      ; res = bucketSize (1 tip) 

line=''
lines=strarr(50000) ; max entries in an event=50000,  enough?
gmin=dblarr(366L*24*60) & gmin(*)=-88.88  ;1-min rainrate (mm/h), 1 file/year, 

eventfs=file_search('EVENTS/*.????', count=nevent)
for ievent=0,nevent-1 do begin

    time=lonarr(14400)  ; max # of rainy minutes in an event=14400 (10days)
    rain=fltarr(14400)

	eventf=eventfs[ievent]
	gauge_id=strmid(eventf,7,7)

	openr, lun_txt, eventf, /get_lun

	n=0L        ; number of stamped rainy minutes in an event
	cmin0=-99   ; cmin  is current minute in an event
    iln=0L      ; number of lines in an event 
    c=0.
    while(not eof(lun_txt)) do begin
        readf,lun_txt,line
        lines[iln]=line
        a=strsplit(line,' ',/extract)
        year=a[0]
        mon=a[1]
        mday=a[2]
        jday=a[3]
        hr=a[4]
        min=a[5]
        sec=a[6]
        tip=a[7] 
        lat=a[8]
        lon=a[9]
        if(iln eq 0) then time1=min*1+hr*60+(jday-1)*24.*60+sec/60.
		; time1 is the first time stamp in the event
        cmin=min*1+hr*60+(jday-1)*24L*60+1   ;current minute in an event
        ;If temporal resolution < 1 min, eg ksc, stj, har, kwa
        ;1-min rain at 12:23 is from 12:22:00 to 12:22:59 if resolution is 1 sec.

        if(cmin gt cmin0) then n=n+1
        c=c+tip*1.	
        time[n-1]=cmin
	rain[n-1]=c     ;unit: mm
	cmin0=cmin
        iln++
    endwhile
    free_lun, lun_txt

    time=time[0:n-1]
    rain=rain[0:n-1]

;  quality control event file for bad data >qc_threshold tips/min
    time2=min+hr*60+(jday-1)*24.*60+sec/60. ;last time stamp in the event
    duration=time2-time1+1./60. ; in minute
    n_tip=round(c/res)          ; number of tips in the event
    event_rate=c/duration

    if (event_rate gt qc_threshold and duration gt 1 and n_tip gt 2) then begin
        for il=0,iln-1 do print, lines[il]
	print, n_tip, ' tips in ', duration, ' minutes ==>',event_rate, " mm/min."
	print, "The above event is discarded!"
	goto,next_event
    endif


;first & last integer minutes of a rain event from time stamps in the event file. 
    min1=time[0]                ; first integer minute
    min2=time[n-1]              ; last  integer minute

    if(min1 lt min2) then begin
        sdrvt=spl_init(time,rain,/double) ; the 2nd derivative of the interpolating function

; Estimate beginning & end minutes (bmin--event start time, emin--event end time)
; Assume there is 0.5 tip of rain filled in the bucket before event starts at bmin 
;        and 0.5 tip of rain filled in the bucket after event ends at emin
; A rain event should be from the 1st half tip to last half tip time
; which is from bmin to emin. Event total rainfall from bmin to emin is 
; the same as from min1 to min2.
; 
; bmin is the minute when extrapolated event rain total reaches 0.5 tip or less
;   (0.5 tip ==>0.0195 inch for HAR or 0.005 inch for others) 
; emin is the minute when extrapolated event rain total reaches 
; (real event total +0.5 tip) or rainrate <=0 
; Thus the event total is preserved. (1st tip -0.5 tip, and last tip +0.5 tip)
; Set rainrate=0 at bmin & emin
; Interpolated rain event is from bmin+1 to emin-1
;
; Set max extrapolation time as bmin0 (=min1-7) and emin0 (=min2+7)
; If bmin runs 7 mins before min1, set bmin=min1-7
; If emin runs 7 mins  after min2, set emin=min2+7
;
; Reference:  Wang et al., 2008, JTECT.

        bmin0=min1-7
        bmin=bmin0
        if(bmin0 lt 0) then begin ; prevent from going to previous year
            bmin0=0 
            bmin=bmin0
        endif

        emin0=min2+7
        emin=emin0
        time22=indgen(emin0-bmin0+1)+bmin0
        spline_rain=dblarr(emin0+1)
        
        spl_rain=spl_interp(time,rain,sdrvt,time22,/double) 
        spline_rain[bmin0:emin0]=spl_rain

        ind=where(spline_rain lt 0, ct)
        if(ct gt 0 ) then spline_rain(ind)=0.

	for i=min1,bmin0+1,-1 do begin
            if( (spline_rain[i] le halftip) or         $ 
                (spline_rain[i] lt spline_rain[i-1]) ) then begin
                bmin=i
		break
            endif
        endfor

	for i=min2,emin0 do begin
            if( (spline_rain[i] gt (spline_rain[min2]+halftip)) or $ 
		(spline_rain[i] lt spline_rain[i-1]) ) then begin
		emin=i 
		break
             endif
        endfor

	; set rainrate=0 at bmin & emin
        ; rainrate=(spline_rain[i]-spline_rain[i-1])*60*25.4 
	if(bmin lt min1) then begin 
            spline_rain[bmin-1]=spline_rain[bmin]
            bmin=bmin+1 ; this'll be used to write out non-zero rates. 
	endif
	if(emin gt min2) then begin
            spline_rain[emin]=spline_rain[emin-1]
            emin=emin-1
        endif

   endif else begin ;one minute event, evenly spread rain into 5 mins interval
        bmin=min1-3
        emin=min2+1
   endelse


; If emin runs into next year, set it as 23:59, Dec 31 of the current year.
   leap=is_leap_yr(year)
   min_in_yr=(365+leap)*24L*60 
   if (emin ge min_in_yr) then emin=min_in_yr-1 ; set to 23:59 Dec 31

; If bmin runs into previous year, set it as 00:00 Jan 1 of the current year.
   if (bmin lt 0) then bmin=0
	
;  get one-minute rain rate
   event_tot_spl=0.D
   for i=bmin, emin do begin 
       if (min1 lt min2) then begin ; normal spline interpolate
           gmin[i]=(spline_rain[i]-spline_rain[i-1])*60 ; mm/hr
           if (gmin[i] lt 0) then gmin[i]=0. 
       endif else begin ; one min event,  evenly spread rain into 5-mins interval
           gmin[i]=rain[n-1]*60/5.0
       endelse
       if (gmin[i] gt 0) then event_tot_spl=event_tot_spl+gmin[i]
   endfor
   
   event_tot_spl=event_tot_spl/60. ; unit: mm 
   bias=event_tot_spl/rain[n-1]

; print,"event=",ievent,"     event_tot_spl=",event_tot_spl, rain[n-1]
; linear interpolation if spline fails (bias<0.5 or bias>1.5)
; linear interpolation from bmin (first time stamp -4 min) to time[n-1](last time stamp)
   if(bias lt 0.5 or bias gt 1.5) then begin
       ;print, "!!! Linear interpolation"
    ; for 1st rainy minute, spread to 4 min ahead
       bmin=time[0]-4
       for i=bmin, time[0] do gmin[i]=rain[0]/(time[0]-bmin+1)*60.

    ; from 2nd rainy minute to last rainy minute,
    ; linearly interpolate rain for each minute
       for j=1,n-1 do begin
           grate=double(rain[j]-rain[j-1])/(time[j]-time[j-1])*60.
           i1=time[j-1]+1
           i2=time[j]
           if(i2 ge min_in_yr) then i2=min_in_yr-1 ;prevent from running into next year.
           for i=i1,i2 do begin
               gmin[i]=grate
               totalsec_to_jdayhhmmss, i*60L, jday, hr, min, sec
;		print "!Linear jday hr:min:sec j=j i=i rain[j] gmin[i]\n"
           endfor
       endfor

       emin=i2      ;linear interpolation from bmin to time[n-1](last time stamp)
       bias=1.      ;linear bias is always 1.
   endif


; write to gmin file
   for i=bmin,emin do begin
       totalsec_to_jdayhhmmss,i*60L, jday, hr, min, sec
       get_calendar_date, year,jday,mon,day 
       tip_number=round(rain[n-1]/res)
       if(gmin[i] gt 0) then printf, lun_gmin, $
            format='(i4,2(2x,i02),2x,i03,2(2x, i02), f8.2, f10.5,1x,f10.5)',$
            year,mon,day,jday,hr,min,gmin[i]/bias, lat, lon
   endfor

   next_event:
   
endfor

free_lun,lun_gmin

end
